/* Hack'n'Slash, A generic MUD engine.
 * Copyright (C) 2000 David Wynn (aka kismet), of the OSOAL tribe
 * send abuse to:    david@osoal.org.nz
 * http requests to: web.osoal.org.nz
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <ctype.h>
#include <string.h>

#include <hacknslash/misc.h>
#include <hacknslash/cmd.h>

CmdList_t * cmd_list_new ( guint         size ) {
  CmdList_t * list = g_new( CmdList_t, 1 );
  cmd_list_init( list, size );
  return list;
}

void        cmd_list_free( CmdList_t *   list ) {
  g_slist_free( list->first );
  g_free( list );
}

void        cmd_list_init( CmdList_t *   list,
			   guint         size ) {
  list->cmd_size = MAX( size, sizeof( Cmd_t ) );
  list->first = list->last = NULL;
}

void        cmd_list_add ( CmdList_t *   cmdlist,
			   Cmd_t *       add ) {
  GSList * new = misc_slist_new( add );
  if( cmdlist->last == NULL )
    cmdlist->first = new;
  else
    cmdlist->last->next = new;
  cmdlist->last = new;
}

Cmd_t *     cmd_find     ( CmdList_t *   cmdlist,
			   const gchar * name,
			   gint          len ) {
  GSList * scan;
  if( !cmdlist || !name || !len )
    return NULL;
  for( scan = cmdlist->first; scan; scan = scan->next ) {
    Cmd_t * cmds = scan->data;
    while( cmds->name ) {
      if( !strncasecmp( name, cmds->name, len ) )
	return cmds;
      cmds = (Cmd_t *)(& ((guchar *)(cmds))[ cmdlist->cmd_size ]); /* wee */
    }
  }
  return NULL;
}

gboolean    cmd_exec     ( CmdList_t *   cmdlist,
			   gpointer      obj,
			   const gchar * line ) {
  gint cmd_i, args_i = -1;
  Cmd_t * cmd;
  gint i, len;

  if( !cmdlist || !line ) 
    return FALSE;

  /* skip leading whitespace */
  for( i = 0; isspace( line[i] ); i++ );
  if( *line == '\0' ) 
    return FALSE;

  for( cmd_i = i; !isspace( line[i] ) && line[i] != '\0'; i++ );
  len = i - cmd_i;

  /* find a command that matches `cmd' */
  if( NULL == (cmd = cmd_find( cmdlist, &(line[cmd_i]), len ) ) || 
      cmd->func == NULL )
    return FALSE;

  /* get arguments */
  if( line[i] != '\0' ) {
    for( ; isspace( line[i] ); i++ );
    if( line[i] != '\0' )
      args_i = i;
  }

  cmd->func( obj, ( args_i > 0 ? &(line[i]) : NULL ), cmd->data );
  return TRUE;
}
