/* Hack'n'Slash, A generic MUD engine.
 * Copyright (C) 2000 David Wynn (aka kismet), of the OSOAL tribe
 * send abuse to:    david@osoal.org.nz
 * http requests to: web.osoal.org.nz
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/* db.c: object file database manipulation. */

#include <stdio.h>
#include <sys/types.h>
#include <dirent.h>
#include <unistd.h>
#include <stdlib.h>

#include <hacknslash/db.h>
#include <hacknslash/misc.h>

inline static
void      _db_id_to_file   ( DB_t * db, guint id, gchar * buf, gint len ) {
  snprintf( buf, len, "%s/%u", db->db_dir, id/db->num_records );
}

static
gint      _db_id_alloc_last( DB_t * db ) {
  gchar buf[128], file[128];
  FILE * f;
  guint id = 1;
  snprintf( file, 128, "%s/last", db->db_dir );
  if( NULL != (f = fopen( file, "r+" ))) {
    gchar * tmp;
    gint i;
    while( 1 ) {
      fgets( buf, 128, f );
      i = strtoul( buf, &tmp, 10 );
      if( buf != tmp )
	break;
    }
    id = i;
    fseek( f, 0, SEEK_SET );
    fprintf( f, "%d\n", id+1 );
    fclose( f );
  }
  return id;
}

gint       db_id_alloc     ( DB_t * db ) {
  gchar buf[128], file[128];
  FILE * f;
  guint id = 0;
  long old, off;
  if( !db ) return 0;
  snprintf( file, 128, "%s/free", db->db_dir );
  f = fopen( file, "r+" );
  off = 0;
  while( 1 ) {
    gchar * tmp;
    gint i;
    old = off;
    off = ftell( f );
    fgets( buf, 128, f );
    if( feof( f ) )
      break;
    i = strtoul( buf, &tmp, 10 );
    if( buf == tmp )
      continue;
    id = i;
  }
  fclose( f );
  /* pop last line off `file' */
  truncate( file, (off_t)(old) );
  if( !id )
    id = _db_id_alloc_last( db );
  return id;
}

void       db_id_free      ( DB_t * db,
                             guint  id ) {
  gchar buf[128], file[128];
  FILE * f;
  guint _id = 0;
  gint done = 0;
  if( !db || !id ) return;
  snprintf( file, 128, "%s/last", db->db_dir );
  if( NULL != (f = fopen( file, "r+" ))) {
    gchar * tmp;
    gint i;
    while( 1 ) {
      fgets( buf, 128, f );
      i = strtoul( buf, &tmp, 10 );
      if( buf != tmp ) 
	break;
    }
    _id = i;
    if( _id == (id+1) ) {
      fseek( f, 0, SEEK_SET );
      fprintf( f, "%d\n", id );
      done = 1;
    }
    fclose( f );
  }
  if( done ) return;
  snprintf( file, 128, "%s/free", db->db_dir );
  f = fopen( file, "r+" );
  if( f ) {
    fseek( f, 0, SEEK_END );
    fprintf( f, "%d\n", id );
    fclose( f );
  }
}

static
void     _db_load_file( DB_t *    db,
			Array_t * array,
			gchar *   file,
			guint     min,
			guint     max ) {
  Block_t * b;
  Object_t * o;
  FILE * f;
  f = fopen( file, "r" );
  if( f ) {
    while( 1 ) {
      if( NULL != (b = block_file_getf( db->def->oclass->name, NULL, f ))) {
        if( b->id ) {
	  gchar * tmp;
	  gint id = strtoul( b->id, &tmp, 10 );
	  if( tmp != b->id && id >= min && id < max ) {
	    o = block_to_object( b, db->def );
	    array_store( array, o, id );
	  }
        }
	block_free( b );
      } else
        break;
    }
    fclose( f );
  }
}

void      db_load_all ( DB_t *    db,
			Array_t * array ) {
  DIR *dp;
  struct dirent *ep;
  gchar dir[256], * start;
  gint end = 0, left, index;
  gchar ch;
  end = snprintf( dir, 256, "%s/", db->db_dir );
  left = 256 - end;
  start = &(dir[end]);
  dp = opendir (dir);
  if (dp != NULL) {
    while( NULL != (ep = readdir(dp)) ) {
      if( 1 == sscanf( ep->d_name, "%u%1c", &index, &ch ) ) {
        misc_strncpy( start, ep->d_name, left );
        _db_load_file( db, array, dir,
		       index * db->num_records, (index+1) * db->num_records );
      }
    }
    closedir (dp);
  }
}

/* */
Object_t * db_load  ( DB_t *      db,
		      guint       id ) {
  gchar file[128];
  gchar tmp[16];
  if( !db || !(db->db_dir) )
    return NULL;
  _db_id_to_file( db, id, file, 128 );
  snprintf( tmp, 16, "%d", id );
  return object_file_get( db->def, tmp, file );
}

/* */
void         db_save  ( DB_t *     db,
			Object_t * obj ) {
  Block_t * b;
  gchar file[128];
  gchar * tmp;
  guint id;
  if( !db || !(db->db_dir) || !b )
    return;
  id = strtoul( obj->id, &tmp, 10 );
  if( obj->id == tmp )
    return;
  _db_id_to_file( db, id, file, 128 );
  b = block_from_object( obj, db->def );
  block_file_insert( b, file );
  block_free( b );
}

/* remove object `id' from the database `db' */
void         db_remove( DB_t *    db,
			guint     id ) {
  gchar tmp[16];
  gchar file[128];
  if( !db || !(db->db_dir) )
    return;
  _db_id_to_file( db, id, file, 128 );
  snprintf( tmp, 16, "%d", id );
  block_file_remove( db->def->oclass->name, tmp, file );
}
