/* Hack'n'Slash, A generic MUD engine.
 * Copyright (C) 2000 David Wynn (aka kismet), of the OSOAL tribe
 * send abuse to:    david@osoal.org.nz
 * http requests to: web.osoal.org.nz
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef __ARRAY_H__
#define __ARRAY_H__

#include <glib.h>
#include <hacknslash/defs.h>

typedef struct Array Array_t;
struct Array {
  guint size;
  GSList ** array;
  
  /* functions to call on data when they're added, removed 
   * (ie, object_ref(), or g_free() or something)
   */
  Func_t on_add;
  Func_t on_remove;
};

/* */
void      array_init   ( Array_t *     array,
			 guint         size,
			 Func_t        on_add,
			 Func_t        on_remove );

Array_t * array_new    ( guint         size,
			 Func_t        on_add,
			 Func_t        on_remove );

void      array_free   ( Array_t *     array );

void      array_clear  ( Array_t *     array );

void      array_store  ( Array_t *     array,
			 gpointer      thing,
			 gint          id );

gpointer  array_get    ( Array_t *     array,
			 gint          id );

void      array_remove ( Array_t *     array,
			 gint          id );



/* */
typedef gboolean (* ArrayForFunc_t) ( gpointer thing,
				      gint     id,
				      gpointer data );

typedef gboolean (* ArrayFindFunc_t)( gpointer thing, 
				      gint     id, 
				      gpointer data );

void      array_foreach( Array_t *       array,
			 ArrayForFunc_t  func,
			 gpointer        data );

gpointer  array_find   ( Array_t *       array,
			 ArrayFindFunc_t func,
			 gpointer        data );

#endif /* __ARRAY_H__ */
