/* Hack'n'Slash, A generic MUD engine.
 * Copyright (C) 2000 David Wynn (aka kismet), of the OSOAL tribe
 * send abuse to:    david@osoal.org.nz
 * http requests to: web.osoal.org.nz
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/* */

#ifndef __BLOCK_H__
#define __BLOCK_H__

#include <glib.h>
#include <stdio.h>

typedef struct BlockDef BlockDef_t;
typedef struct BlockVar BlockVar_t;
typedef struct Block    Block_t;

#include <hacknslash/object.h>

/* the BlockDef struct is used to describe how to create an object
 * from it's text source.
 */
struct BlockDef {
  ObjectClass_t *    oclass;

  /* methods for conterting object to/from blocks */
  gboolean (* add_var) ( Object_t * obj, BlockVar_t * var );
  GSList * (* get_vars)( Object_t * obj );

  GSList * blocks;
};



/* */
struct BlockVar {
  gchar * label;
  gchar * value;
};

BlockVar_t * block_var_new    ( gchar *      label,
				gchar *      value );

void         block_var_free   ( BlockVar_t * var );



/* */
struct Block {
  gchar * label;
  gchar * id;
  GSList * blocks, * blocks_last;
  GSList * vars, * vars_last;
};

Block_t *    block_new        ( gchar *      label,
				gchar *      id );

void         block_free       ( Block_t *    block );

/* remove all blocks/vars from `block' */
void         block_clear      ( Block_t *    block );

void         block_add_block  ( Block_t *    block,
				Block_t *    subblock );

Block_t *    block_get_block  ( Block_t *    block,
				gchar *      label,
				gchar *      id );

/* */
void         block_add_var    ( Block_t *    block,
				BlockVar_t * var );

BlockVar_t * block_get_var    ( Block_t *    block,
				gchar *      label );

/* */ 
Block_t *    block_file_getf  ( gchar *      label,
				gchar *      id,
				FILE *       f );

Block_t *    block_file_get   ( gchar *      label,
				gchar *      id,
				gchar *      file );

void         block_file_write ( Block_t *    block,
 				FILE *       f );

void         block_file_insert( Block_t *    src,
				gchar *      file );

void         block_file_remove( gchar *      label,
				gchar *      id,
				gchar *      file );

/* compose an object from `block' using `def' */
Object_t *   block_to_object  ( Block_t *    block,
				BlockDef_t * def );

/* decompose `object' using `def' into a block */
Block_t *    block_from_object( Object_t *   object,
				BlockDef_t * def );

#endif /* __BLOCK_H__ */

