/* Hack'n'Slash, A generic MUD engine.
 * Copyright (C) 2000 David Wynn (aka kismet), of the OSOAL tribe
 * send abuse to:    david@osoal.org.nz
 * http requests to: web.osoal.org.nz
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef __CMD_H__
#define __CMD_H__

#include <glib.h>

/* */
typedef void (* CmdFunc_t)( gpointer       obj,
			    const gchar *  args,
			    gpointer       data );

/* */
typedef struct CmdList CmdList_t;
struct CmdList {
  guint cmd_size; /* size of each Cmd struct */
  GSList * first, * last;
};

/* */
typedef struct Cmd Cmd_t;
struct Cmd {
  gchar * name;
  CmdFunc_t func;
  gpointer data;
};

CmdList_t * cmd_list_new ( guint         size );
void        cmd_list_free( CmdList_t *   cmdlist );
void        cmd_list_init( CmdList_t *   cmdlist,
			   guint         size );

/* add reference of `add' to `cmdlist', `add' is not copied! 
 * so must remain in existance while `cmdlist' is in use.
 */
void        cmd_list_add ( CmdList_t *   cmdlist,
			   Cmd_t *       add );

Cmd_t *     cmd_find     ( CmdList_t *   cmdlist,
			   const gchar * name,
			   gint          len );

/* splits `line' into a cmd (first word) and argument list
 * (rest of string beginning with second word), finds the cmd
 * that it matches and execs it.  `line' is not clobbered.
 * returns TRUE if a matching command was found, FALSE otherwise
 */
gboolean    cmd_exec     ( CmdList_t *   cmdlist,
			   gpointer      obj,
			   const gchar * line );

#endif /* __CMD_H__ */


