/* Hack'n'Slash, A generic MUD engine.
 * Copyright (C) 2000 David Wynn (aka kismet), of the OSOAL tribe
 * send abuse to:    david@osoal.org.nz
 * http requests to: web.osoal.org.nz
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/* client server.
 *   the client server is dumb and only parses enough telnet commands
 *   to handle responses to request for server-side echoing
 *   when a write error occurs, the client is disconnected during the
 *   next service.
 */

#ifndef __SERVER_H__
#define __SERVER_H__

#include <glib.h>

#include <hacknslash/defs.h>
#include <hacknslash/object.h>
#include <hacknslash/colors.h>

/* size of line buffer (in characters, add one for the null terminator) */
#define SOCK_LINE_SIZE 255

typedef struct      Sock             Sock_t;
typedef struct      SockClass        SockClass_t;

typedef void     (* SockFunc_t)    ( Sock_t *, gpointer data );
typedef gboolean (* SockFindFunc_t)( Sock_t *, gpointer data );

struct SockClass {
  ObjectClass_t parent;
};

/* returns a pointer to the class for type `Sock' */
SockClass_t * sock_class();

struct Sock {
  Object_t parent;

  gint fd;
  gint read_handle;

  /* the currently read line */
  gchar line[ SOCK_LINE_SIZE+1 ];
  guint line_i;

  guint colors : 1; /* dont send color escape sequences to the client */
  guint mute   : 1; /* dont send anything to the client */

  /* put character handler (for intenal use) */
  void (* putchar)( Sock_t * sock, guchar ch );
};



/* objects used to describe sockets are created with `sclass' */
void     server_init               ( SockClass_t * sclass );

void     server_open               ( gint          port,
				     gint          max_clients );

void     server_close              ();

/* tell client that the server will handle echoing */
void     server_echo               ( Sock_t *   sock,
				     gboolean   echo );

/* a mute client cannot be sent anything with server_send[f]
 * server_write() will still work though
 */
void     server_mute               ( Sock_t *   sock,
				     gboolean   mute );

/* write telnet Are You There command to `sock' */
void     server_ping               ( Sock_t * sock );

/* send `data' to `sock' */
void     server_write              ( Sock_t *   sock,
				     gpointer   data,
				     guint      len );

/* calculate length of `text' and send it to sock via server_write() */
void     server_send               ( Sock_t *   sock,
				     gint       col,
				     gchar *    text );

void     server_sendf              ( Sock_t *   sock,
				     gint       col,
				     gchar *    format,... );

/* send `text' to all connected clients */
void     server_broadcast          ( gint       col,
				     gchar *    text );

void     server_broadcastf         ( gint       col,
				     gchar *    format,... );

/* call `func' with `data' on each connected socket */
void     server_foreach            ( SockFunc_t func,
				     gpointer   data );

Sock_t * server_find               ( SockFindFunc_t func,
				     gpointer       data );

/* disconnect `sock' */
void     server_disconnect         ( Sock_t *   sock );

Sock_t * server_get_sock           ( gint       id );

/* call `func' with a new sock and `data' whenever a client
 * connects.
 */
void     server_set_connect_func   ( SockFunc_t func,
				     gpointer   data );

void     server_set_read_func      ( SockFunc_t func,
				     gpointer   data );

void     server_set_disconnect_func( SockFunc_t func,
				     gpointer   data );

#endif /* __SERVER_H__ */
