/* Hack'n'Slash, A generic MUD engine.
 * Copyright (C) 2000 David Wynn (aka kismet), of the OSOAL tribe
 * send abuse to:    david@osoal.org.nz
 * http requests to: web.osoal.org.nz
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/* io.c: wrappers for the glib io monitoring functions. this code is
 *    basically copied and modified from the implementation of 
 *   `gdk_input_add()' from GDK (that came with * GTK 1.2.6)
 */

#include <hacknslash/io.h>

typedef struct _IOClosure _IOClosure_t;
struct _IOClosure {
  GIOCondition cond;
  IOFunc_t     func;
  gpointer     data;
  Func_t       destroy;
};

static
gboolean _io_invoke( GIOChannel *   source,
		     GIOCondition   cond,
		     _IOClosure_t * closure ) {
  if( cond & closure->cond )
    closure->func( g_io_channel_unix_get_fd( source ), cond, closure->data );
  return TRUE;
}

static
void _io_destroy( _IOClosure_t * closure ) {
  if( closure->destroy )
    closure->destroy( closure->data );
  g_free( closure );
}

gint io_hook( gint     fd, 
	      gint     how, 
	      IOFunc_t func, 
	      gpointer data, 
	      Func_t   destroy ) {
  guint result;
  _IOClosure_t *closure = g_new( _IOClosure_t, 1 );
  GIOChannel * channel;

  closure->func = func;
  closure->cond = how;
  closure->data = data;
  closure->destroy = destroy;

  channel = g_io_channel_unix_new( fd );
  result = g_io_add_watch_full( channel, G_PRIORITY_DEFAULT, 
				(GIOCondition)how,
                                (GIOFunc)_io_invoke,
                                closure, 
				(GDestroyNotify)_io_destroy );
  g_io_channel_unref(channel);
  return result;
}

void io_unhook   ( gint handle ) {
  g_source_remove( handle );
}





