/* Hack'n'Slash, A generic MUD engine.
 * Copyright (C) 2000 David Wynn (aka kismet), of the OSOAL tribe
 * send abuse to:    david@osoal.org.nz
 * http requests to: web.osoal.org.nz
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/* misc.c:
 *   this is a big mess, it's where all the homeless, utility code hangs out.
 */

#include <string.h>
#include <ctype.h>
#include <stdlib.h>

#include <hacknslash/misc.h>
#include <hacknslash/token.h>

gint    misc_strcmp( const char * str1, const char * str2 ) {
  if( !str1 ) {
    if( !str2 )
      return 0;
    return -1;
  }
  if( !str2 )
    return 1;
  return strcmp( str1, str2 );
}

void    misc_strncpy ( gchar * dest, const gchar * src, gint len ) {
  strncpy( dest, src, len-1 );
  dest[len-1] = '\0';
}

gint    misc_strindex( const gchar * str, const gchar ** strv ) {
  gint i;
  for( i = 0; strv[i] != NULL; i++ )
    if( !strcmp( str, strv[i] ) )
      return i;
  return -1;
}

gint    misc_strnlen ( const gchar * str, gint len ) {
  int i;
  for( i = 0; i < len && str[i]; i++ );
  return i;
}

gchar * misc_strndup ( const gchar * str, gint len ) {
  gchar * s;
  if( !str )
    return NULL;
  len = misc_strnlen( str, len );
  s = g_new( gchar, len+1 );
  memcpy( s, str, len );
  s[len] = 0;
  return s;
}

GSList * misc_strsplitl( const gchar * str,
			 const gchar * delim,
			 const gchar * quote,
			 gint *  max ) {
  gchar * token, * s = str;
  gint len, num;
  
  GSList * first, * list, * last = NULL;

  if( !s || max == NULL || *max <= 0 )
    return NULL;

  num = 0;
  while( num < *max &&
	 NULL != (token = token_get( s, delim, quote, &len, &s ) ) ) {
    list = g_slist_alloc();
    list->data = misc_strndup( token, len );
    list->next = NULL;
    if( last )
      last->next = list;
    else 
      first = list;
    last = list;
    num++;
  }

  *max = num;

  return list;
}

gchar ** misc_strsplit( const gchar * str,
			const gchar * delim,
			const gchar * quote,
			gint *  max ) {

  GSList * list = NULL;
  gchar ** strv;
  gint i;

  list = misc_strsplitl( str, delim, quote, max );
  if( list == NULL )
    return NULL;

  strv = g_new( gchar *, *max );
  (*max)--;
  strv[*max] = NULL;

  for( i = 0; list != NULL && i < *max; list = list->next, i++ )
    strv[i] = list->data;

  g_slist_free( list );

  return strv;
}

gchar ** misc_strvdup( gchar ** strv ) {
  gint num;
  gchar ** newv;

  if( strv == NULL )
    return NULL;

  for( num = 1; strv[num]; num++ );

  newv = g_new( gchar *, num );
  newv[num] = 0;

  for( num--; num > 0; num-- )
    newv[num] = strv[num];

  return newv;
}

void misc_strtobool( gboolean * flag, gchar * str ) {
  if( !strcasecmp( str, "true" ) )
    *flag = TRUE;
  else if( !strcasecmp( str, "false" ) )
    *flag = FALSE;
}

/* read a non-comment line from `f' */
gboolean misc_getline( gchar * buf, gint len, FILE * f ) {
  gint i;

  while( 1 ) {
    fgets( buf, len, f );
    if( feof( f ) )
      break;

    for( i = 0; buf[i]; i++ ) {
      if( buf[i] == '#' )
	break;
      if( !isspace( buf[i] ) )
	return 1;
    }
  }

  return 0;
}

gchar * misc_indent( gint indent ) {
  return ( indent ? g_strnfill( indent*2, ' ' ) : g_strdup( "" ) );
}

void misc_fcopy( FILE * f, FILE * g, gint len ) {
  gchar buf[256];
  gint i;
  if( len >= 0 ) {
    while( 1 ) {
      i = fread( buf, 1, MIN( 256, len ), f );
      if( i <= 0 )
	break;
      fwrite( buf, 1, i, g );
      len-=i;
    }
  } else {
    while( 1 ) {
      i = fread( buf, 1, 256, f );
      if( i <= 0 )
	break;
      fwrite( buf, 1, i, g );
    }
  }
}

void misc_fgets( gchar * str,
	       gint    len,
	       FILE *  f ) {
  gint i;
  len--;
  for( i = 0; i < len; i++ ) {
    str[i] = fgetc( f );
    if( feof( f ) )
      break;
  }
  str[i] = '\0';
}

void misc_findent( gint len,
		   FILE * f ) {
  gint i;
  for( i = 0; i < len; i++ )
    fputc( ' ', f );
}

void misc_dtoa( gchar * buf, gint len, gdouble d ) {
  gint i;
  i = snprintf( buf, len, "%f", d ) - 1;
  while( i > 0 ) {
    if( buf[i] == '0' ) {
      buf[i] = 0;
      i--;
      continue;

    } else if( buf[i] == '.' )
      buf[i] = 0;

    break;
  }
}

void misc_fputd( gdouble d,
		 FILE * f ) {
  gchar buf[16];
  misc_dtoa( buf, 16, d );
  fputs( buf, f );
}

/* tricky to explain... */
gboolean misc_gmatch( GSList * mask,
		      GSList * test ) {
  gchar * s1, * s2;
  gint res = -1;
  gint tmp;
  gint neg;
  GSList * slist;

  for( ; test; test = test->next ) {
    s1 = (gchar *)(test->data);
    tmp = -1;
    for( slist = mask; slist; slist = slist->next ) {
      s2 = (gchar *)(slist->data);
      if( s2[0] == '!' ) {
	neg = 1;
	s2++;
      } else
	neg = 0;

      if( !strcmp( s2, "ALL" ) ) {
	if( tmp )
	  tmp = 1;
      } else if( !strcmp( s1, s2 ) ) {
	if( neg )
	  tmp = 0;
	else
	  tmp = 1;
      }
    }
    if( tmp != -1 ) {
      if( res && !tmp )
	res = 0;
      else if( res < 0 && tmp )
	res = 1;
    }
  }
  return (res == 1 ? TRUE : FALSE);
}


void misc_slist_unlink( GSList ** link,
		      GSList *  trail,
		      GSList ** first ) {
  if( trail ) 
    trail->next = (*link)->next;
  else 
    *first = (*link)->next;
  trail = (*link)->next;
  g_slist_free_1( *link );
  *link = trail;
}

/* returns a new slist with the elements in `slist', with jumbled order. 
 * the old list isn't freed. */
GSList * misc_slist_jumble( GSList * slist ) {
  GSList * scan, * new = NULL;
  gint i;
  for( scan = slist, i = 1; 
       scan;
       scan = scan->next, i++ )
    new = g_slist_insert( new, scan->data, random()%i );
  return new;
}

/* count the number of elements in `slist' */
gint     misc_slist_count( GSList * slist ) {
  gint num;
  for( num = 0; slist; slist = slist->next, num++ );
  return num;
}

/* choose `choose' random elements from `slist' and return them in a jumbled
 * order.
 */
GSList * misc_slist_choose( GSList * slist, 
			    gint choose ) {
  GSList * scan, * new;
  gint i;
  gint num;
  num = misc_slist_count( slist );
  if( !num )
    return NULL;
  choose = MIN( num, choose );
  for( new = NULL, scan = slist, i = choose; 
       scan && i > 0;
       scan = scan->next, num-- ) {
    if( random()%num < i ) {
      i--;
      new = g_slist_insert( new, scan->data, random()%(choose-i) );
    }
  }
  return new;
}

GSList * misc_slist_reverse( GSList * slist ) {
  GSList * new = NULL;
  while( slist ) {
    new = g_slist_prepend( new, slist->data );
    slist = slist->next;
  }
  return new;
}

GSList * misc_slist_new( gpointer data ) {
  GSList * new;
  new = g_slist_alloc();
  new->data = data;
  new->next = NULL;
  return new;
}
